// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
import { describe, expect, it } from 'vitest';
import { Vector2Utils, VECTOR3_LEFT, VECTOR3_UP, Vector3Utils } from './coreHelpers.js';
describe('Vector3 operations', () => {
    const v1 = { x: 1, y: 2, z: 3 };
    const v2 = { x: 4, y: 5, z: 6 };
    it('successfully compares vectors', () => {
        const v3 = { x: 1, y: 2, z: 3 };
        expect(Vector3Utils.equals(v1, v3)).toBe(true);
        expect(Vector3Utils.equals(v1, v2)).toBe(false);
    });
    it('successfully adds vectors and returns a new vector', () => {
        const result = Vector3Utils.add(v1, v2);
        expect(result).toEqual({ x: 5, y: 7, z: 9 });
        expect(result).not.toBe(v1);
    });
    it('successfully subtracts vectors and returns a new vector', () => {
        const result = Vector3Utils.subtract(v1, v2);
        expect(result).toEqual({ x: -3, y: -3, z: -3 });
        expect(result).not.toBe(v1);
    });
    it('successfully scales a vector and returns a new vector', () => {
        const result = Vector3Utils.scale(v1, 2);
        expect(result).toEqual({ x: 2, y: 4, z: 6 });
        expect(result).not.toBe(v1);
    });
    it('successfully computes the dot product of a vector', () => {
        const result = Vector3Utils.dot(v1, v2);
        expect(result).toBe(32);
    });
    it('successfully computes the dot product of a vector with a 0 vector', () => {
        const result = Vector3Utils.dot(v1, { x: 0, y: 0, z: 0 });
        expect(result).toBe(0);
    });
    it('successfully computes the cross product of a vector and returns a new vector', () => {
        const result = Vector3Utils.cross(v1, v2);
        expect(result).toEqual({ x: -3, y: 6, z: -3 });
        expect(result).not.toBe(v1);
        expect(result).not.toBe(v2);
    });
    it('returns a zero vector for a cross product of parallel vectors', () => {
        const result = Vector3Utils.cross({ x: 3, y: 0, z: 0 }, { x: 7, y: 0, z: 0 });
        expect(result).toEqual({ x: 0, y: 0, z: 0 });
    });
    it('returns a zero vector for a cross product of with a zero vector', () => {
        const result = Vector3Utils.cross(v1, { x: 0, y: 0, z: 0 });
        expect(result).toEqual({ x: 0, y: 0, z: 0 });
        expect(result).not.toBe(v1);
    });
    it('returns the unit z vector for a cross product of unit x and unit y vectors', () => {
        const result = Vector3Utils.cross({ x: 1, y: 0, z: 0 }, { x: 0, y: 1, z: 0 });
        expect(result).toEqual({ x: 0, y: 0, z: 1 });
    });
    it('calculates the magnitude', () => {
        const result = Vector3Utils.magnitude(v1);
        expect(result).toBeCloseTo(3.74, 2);
    });
    it('calculates the distance between two vectors', () => {
        const result = Vector3Utils.distance(v1, v2);
        expect(result).toBeCloseTo(5.2, 2);
    });
    it('computes the floor of the vector', () => {
        const input = { x: 1.33, y: 2.14, z: 3.55 };
        const expected = { x: 1, y: 2, z: 3 };
        expect(Vector3Utils.floor(input)).toEqual(expected);
    });
    it('computes the floor of negative vectors', () => {
        const input = { x: -1.33, y: -2.14, z: -3.55 };
        const expected = { x: -2, y: -3, z: -4 };
        expect(Vector3Utils.floor(input)).toEqual(expected);
    });
    it('normalizes the vector', () => {
        const result = Vector3Utils.normalize(v1);
        expect(result.x).toBeCloseTo(0.27, 2);
        expect(result.y).toBeCloseTo(0.53, 2);
        expect(result.z).toBeCloseTo(0.8, 2);
    });
    it('converts a vector to a string with default options', () => {
        const vector = { x: 1, y: 2, z: 3 };
        const expectedString = '1.00, 2.00, 3.00';
        expect(Vector3Utils.toString(vector)).toBe(expectedString);
        expect(Vector3Utils.toString(vector, undefined)).toBe(expectedString);
        expect(Vector3Utils.toString(vector, { decimals: undefined, delimiter: undefined })).toBe(expectedString);
    });
    it('converts a vector to a string with overridden options', () => {
        const vector = { x: 1.23456789, y: 2.99, z: 3 };
        const expectedString1 = '1.2346|2.9900|3.0000'; // toFixed performs rounding
        expect(Vector3Utils.toString(vector, { decimals: 4, delimiter: '|' })).toBe(expectedString1);
        const expectedString2 = '1|3|3';
        expect(Vector3Utils.toString(vector, { decimals: 0, delimiter: '|' })).toBe(expectedString2);
        const expectedString3 = '1, 3, 3';
        expect(Vector3Utils.toString(vector, { decimals: 0 })).toBe(expectedString3);
        const expectedString4 = '1.23|2.99|3.00';
        expect(Vector3Utils.toString(vector, { delimiter: '|' })).toBe(expectedString4);
    });
    it('converts a vector2 to a string with default options', () => {
        const vector = { x: 1, y: 2 };
        const expectedString = '1.00, 2.00';
        expect(Vector2Utils.toString(vector)).toBe(expectedString);
        expect(Vector2Utils.toString(vector, undefined)).toBe(expectedString);
        expect(Vector2Utils.toString(vector, { decimals: undefined, delimiter: undefined })).toBe(expectedString);
    });
    it('converts a vector2 to a string with overridden options', () => {
        const vector = { x: 1.23456789, y: 2.99 };
        const expectedString1 = '1.2346|2.9900'; // toFixed performs rounding
        expect(Vector2Utils.toString(vector, { decimals: 4, delimiter: '|' })).toBe(expectedString1);
        const expectedString2 = '1|3';
        expect(Vector2Utils.toString(vector, { decimals: 0, delimiter: '|' })).toBe(expectedString2);
        const expectedString3 = '1, 3';
        expect(Vector2Utils.toString(vector, { decimals: 0 })).toBe(expectedString3);
        const expectedString4 = '1.23|2.99';
        expect(Vector2Utils.toString(vector, { delimiter: '|' })).toBe(expectedString4);
    });
    describe('fromString', () => {
        it('creates a vector3 from a string', () => {
            const inputString = '1.00, 2.00, 3.00';
            const expectedVector = { x: 1, y: 2, z: 3 };
            expect(Vector3Utils.fromString(inputString)).toEqual(expectedVector);
        });
        it('for a vector3 returns undefined for invalid formatting', () => {
            const invalidTooManyNumbers = '1.00, 2.00, 3.00, 345, 234234';
            const invalidIncorrectDelimiter = '1.00+ 2.00+ 3.00+ 345+ 234234';
            const invalidNonNumericElements = '1.00, wrong, 3.00, 3fgdf7, 234234';
            expect(Vector3Utils.fromString(invalidTooManyNumbers)).toBeUndefined();
            expect(Vector3Utils.fromString(invalidIncorrectDelimiter)).toBeUndefined();
            expect(Vector3Utils.fromString(invalidNonNumericElements)).toBeUndefined();
        });
        it('creates a vector3 from a string with an overridden delimiter', () => {
            const inputString = '1.00+ 2.00+ 3.00';
            const expectedVector = { x: 1, y: 2, z: 3 };
            expect(Vector3Utils.fromString(inputString, '+')).toEqual(expectedVector);
        });
        it('creates a vector2 from a string', () => {
            const inputString = '1.00, 2.00';
            const expectedVector = { x: 1, y: 2 };
            expect(Vector2Utils.fromString(inputString)).toEqual(expectedVector);
        });
        it('for a vector2 returns undefined for invalid formatting', () => {
            const invalidTooManyNumbers = '1.00, 2.00, 3.00, 345, 234234';
            const invalidIncorrectDelimiter = '1.00+ 2.00';
            const invalidNonNumericElements = 'fgdf7, 234234';
            expect(Vector2Utils.fromString(invalidTooManyNumbers)).toBeUndefined();
            expect(Vector2Utils.fromString(invalidIncorrectDelimiter)).toBeUndefined();
            expect(Vector2Utils.fromString(invalidNonNumericElements)).toBeUndefined();
        });
        it('creates a vector2 from a string with an overridden delimiter', () => {
            const inputString = '1.00+ 2.00';
            const expectedVector = { x: 1, y: 2 };
            expect(Vector2Utils.fromString(inputString, '+')).toEqual(expectedVector);
        });
    });
    describe('clamp', () => {
        const v = { x: 1, y: 1, z: 3 };
        const minVec = { x: 0, y: 1.5 };
        const maxVec = { x: 2, z: 2.5 };
        it('clamps with defaults (no min or max)', () => {
            const result = Vector3Utils.clamp(v);
            expect(result).toEqual({ x: 1, y: 1, z: 3 });
        });
        it('clamps properly with both min and max', () => {
            const result = Vector3Utils.clamp(v, { min: minVec, max: maxVec });
            expect(result).toEqual({ x: 1, y: 1.5, z: 2.5 });
        });
        it('clamps with min only', () => {
            const result = Vector3Utils.clamp(v, { min: minVec });
            expect(result).toEqual({ x: 1, y: 1.5, z: 3 });
        });
        it('clamps with max only', () => {
            const result = Vector3Utils.clamp(v, { max: maxVec });
            expect(result).toEqual({ x: 1, y: 1, z: 2.5 });
        });
        it('clamp with zero vector and positive mins and negative max', () => {
            const vZero = { x: 0, y: 0, z: 0 };
            const min = { y: 1.5 };
            const max = { z: -2.5 };
            const result = Vector3Utils.clamp(vZero, { min, max });
            expect(result).toEqual({ x: 0, y: 1.5, z: -2.5 });
        });
        // Test clamp function with large vector
        const vLarge = { x: 1e6, y: 1e6, z: 1e6 };
        it('clamp with large vector', () => {
            const result = Vector3Utils.clamp(vLarge, { min: minVec, max: maxVec });
            expect(result).toEqual({ x: 2, y: 1e6, z: 2.5 });
        });
    });
    it('calculates the lerp halfway between two vectors', () => {
        const result = Vector3Utils.lerp(v1, v2, 0.5);
        expect(result).toEqual({ x: 2.5, y: 3.5, z: 4.5 });
    });
    it('calculates the slerp halfway between two vectors', () => {
        const vecA = { x: 1, y: 0, z: 0 };
        const vecB = { x: 0, y: -1, z: 0 };
        const result = Vector3Utils.slerp(vecA, vecB, 0.5);
        expect(result.x).toBeCloseTo(0.7071, 3);
        expect(result.y).toBeCloseTo(-0.7071, 3);
        expect(result.z).toBeCloseTo(0);
    });
    it('calculates two vectors multiplied together', () => {
        const result = Vector3Utils.multiply(v1, v2);
        expect(result).toEqual({ x: 4, y: 10, z: 18 });
    });
    describe('Vector3 rotation functions', () => {
        it(`calculates a vector rotated along the x axis`, () => {
            const result = Vector3Utils.rotateX(VECTOR3_UP, Math.PI / 2);
            expect(result.x).toBeCloseTo(0);
            expect(result.y).toBeCloseTo(0);
            expect(result.z).toBeCloseTo(1);
        });
        it(`calculates a vector rotated along the y axis`, () => {
            const result = Vector3Utils.rotateY(VECTOR3_LEFT, Math.PI / 2);
            expect(result.x).toBeCloseTo(0);
            expect(result.y).toBeCloseTo(0);
            expect(result.z).toBeCloseTo(1);
        });
        it(`calculates a vector rotated along the z axis`, () => {
            const result = Vector3Utils.rotateZ(VECTOR3_UP, Math.PI / 2);
            expect(result.x).toBeCloseTo(-1);
            expect(result.y).toBeCloseTo(0);
            expect(result.z).toBeCloseTo(0);
        });
    });
    describe('Vector3 add partial', () => {
        it(`add a single axis`, () => {
            const result = Vector3Utils.add(VECTOR3_UP, { z: 4 });
            expect(result.x).toBeCloseTo(0);
            expect(result.y).toBeCloseTo(1);
            expect(result.z).toBeCloseTo(4);
        });
        it(`add two axis`, () => {
            const result = Vector3Utils.add(VECTOR3_UP, { z: 7, x: 2 });
            expect(result.x).toBeCloseTo(2);
            expect(result.y).toBeCloseTo(1);
            expect(result.z).toBeCloseTo(7);
        });
        it(`add all three axis`, () => {
            const result = Vector3Utils.add(VECTOR3_UP, { x: 8, y: 2, z: 3 });
            expect(result.x).toBeCloseTo(8);
            expect(result.y).toBeCloseTo(3);
            expect(result.z).toBeCloseTo(3);
        });
    });
    describe('Vector3 subtract partial', () => {
        it(`subtract a single axis`, () => {
            const result = Vector3Utils.subtract(VECTOR3_UP, { z: 4 });
            expect(result.x).toBeCloseTo(0);
            expect(result.y).toBeCloseTo(1);
            expect(result.z).toBeCloseTo(-4);
        });
        it(`subtract two axis`, () => {
            const result = Vector3Utils.subtract(VECTOR3_UP, { z: 7, x: 2 });
            expect(result.x).toBeCloseTo(-2);
            expect(result.y).toBeCloseTo(1);
            expect(result.z).toBeCloseTo(-7);
        });
        it(`subtract all three axis`, () => {
            const result = Vector3Utils.subtract(VECTOR3_UP, { x: 8, y: 2, z: 3 });
            expect(result.x).toBeCloseTo(-8);
            expect(result.y).toBeCloseTo(-1);
            expect(result.z).toBeCloseTo(-3);
        });
    });
});
//# sourceMappingURL=coreHelpers.test.js.map